/**
 * Author: Peder A. Nielsen
 * Date: 03.02.15
 * Company: Making Waves
 */
// UMD dance - https://github.com/umdjs/umd
//https://gist.github.com/simonsmith/4353587
//https://github.com/jquery-boilerplate/jquery-boilerplate/blob/master/src/jquery.boilerplate.js
//https://github.com/jquery-boilerplate/jquery-boilerplate/wiki/jQuery-boilerplate-and-demo
//https://github.com/acanimal/tagger.js/blob/master/tagger.js
//http://acuriousanimal.com/blog/2013/01/15/things-i-learned-creating-a-jquery-plugin-part-i/

!function (root, factory) {
    if (typeof define === 'function' && define.amd) {
        define(['jquery'], factory);
    } else {
        factory(root.jQuery);
    }
}(this, function ($) {
    'use strict';

    // Create the defaults once
    var Plugin,
        pluginName = "pederPlugin",
        dataKey = "plugin_" + pluginName,
        defaults = {
            bgColor: "red"
        };

    // The actual plugin constructor
    Plugin = function (element, options) {
        this.$element = $(element);
        // jQuery has an extend method which merges the contents of two or
        // more objects, storing the result in the first object. The first object
        // is generally empty as we don't want to alter the default options for
        // future instances of the plugin
        this.settings = $.extend({}, defaults, options);
        this._defaults = defaults;
        this._name = pluginName;
        this.init();
    }

    // Avoid Plugin.prototype conflicts
    $.extend(Plugin.prototype, {
        constructor: Plugin,
        init: function () {
            // Place initialization logic here
            // You already have access to the DOM element and
            // the options via the instance, e.g. this.element
            // and this.settings
            // you can add more functions like the one below and
            // call them like so: this.yourOtherFunction(this.element, this.settings).
            //console.log("init");
            this._setBgColor();
        },
        _setBgColor: function () {
            this.$element.css('background-color', this.settings.bgColor);
        },
        myPublicMethod: function () {
            // some logic
            this.$element.css('background-color', 'blue');
        }
    });

    // A really lightweight plugin wrapper around the constructor,
    // preventing against multiple instantiations
    $.fn[pluginName] = function (options) {
        if (options === undefined || typeof options === 'object') {
            return this.each(function () {
                if (!$.data(this, dataKey)) {
                    // Create a new instance for each element in the matched jQuery set
                    // Also save the instance so it can be accessed later to use methods/properties etc
                    // e.g.
                    //    var instance = $('.element').data('plugin_pederPlugin');
                    //    instance.someMethod();
                    $.data(this, dataKey, new Plugin(this, options));
                }
            });
        } else if (typeof options === 'string' && options[0] !== '_' && options !== 'init') {
            // Invoke the specified method on each selected element
            //e.g
            //  var p = $('p');
            //  p.pederPlugin('myPublicMethod');
            return this.each(function () {
                var instance = $.data(this, dataKey);
                if (instance instanceof Plugin && typeof instance[options] === 'function') {
                    instance[options].apply(instance, Array.prototype.slice.call(arguments, 1));
                }
            });
        }
    };

    // Expose defaults and Constructor (allowing overriding of prototype methods for example)
    $.fn[pluginName].defaults = Plugin.defaults = defaults;
    $.fn[pluginName].Plugin = Plugin;

    return Plugin;
});


// Usage example
//require(['jquery', 'jquery.plugin'], function($) {
//    $('.test li').plugin({
//        test: 'option1',
//        test2: 'option2'
//    });
//});